@@ -5,12 +5,13 @@
 import re
 from urlparse import urlparse
 
-from formencode import validators, Invalid, Schema
+from formencode import validators, Schema
 
 valid_username = re.compile(r'^[.\w-]{1,32}$', re.U)
 valid_password = re.compile(r'^(.){7,200}$', re.U)
 valid_digest = re.compile(r'^[a-f0-9]{28,56}$')
 
+
 class Username(validators.UnicodeString):
     """Validates that the user input matches ``valid_username``, strips and
       coerces to lowercase.
@@ -73,7 +74,7 @@ class Password(validators.UnicodeString):
 
 
 class Email(validators.Email):
-    """Patch ``validators.Email`` with ``validators.UnicodeString``s 
+    """Patch ``validators.Email`` with ``validators.UnicodeString``s
       ``_to_python`` method.
 
       If it isn't valid, raises an exception::
@@ -87,7 +88,8 @@ class Email(validators.Email):
           ...
           Invalid: The domain portion of the email address is invalid (the portion after the @: baz)
 
-      Note that when used with ``resolve_domain=True`` it must be a real domain::
+      Note that when used with ``resolve_domain=True`` it must be a real
+      domain::
 
           >>> Email(resolve_domain=True).to_python('a@b.com')
           Traceback (most recent call last):
@@ -106,7 +108,6 @@ class Email(validators.Email):
         return value.strip().lower() if value else value
 
 
-
 class RequestPath(validators.UnicodeString):
     """Valid `/request/path`.
 
@@ -170,7 +171,6 @@ class Digest(validators.UnicodeString):
             raise validators.Invalid(msg, value, state)
 
 
-
 class UniqueUsername(Username):
     """A ``Username`` that hasn't already been taken.
 
@@ -254,13 +254,13 @@ class UniqueEmail(Email):
             raise validators.Invalid(msg, value, state)
 
 
-
 class FlexibleSchema(Schema):
     """``formencode.Schema`` that defaults to allow and filter extra fields."""
 
     filter_extra_fields = True
     allow_extra_fields = True
 
+
 class Signup(FlexibleSchema):
     """Form fields to render and validate for signup."""
 
@@ -270,17 +270,19 @@ class Signup(FlexibleSchema):
     confirm = Password(not_empty=True)
     chained_validators = [
         validators.FieldsMatch(
-            'password', 
+            'password',
             'confirm'
         )
     ]
 
+
 class Authenticate(FlexibleSchema):
     """Form fields to validate when authenticating over XHR."""
 
     username = Username(not_empty=True)
     password = Password(not_empty=True)
 
+
 class Login(FlexibleSchema):
     """Form fields to render and validate for login."""
 
@@ -296,7 +298,7 @@ class ChangePassword(FlexibleSchema):
     new_confirm = Password(not_empty=True)
     chained_validators = [
         validators.FieldsMatch(
-            'new_password', 
+            'new_password',
             'new_confirm'
         )
     ]
