@@ -6,22 +6,24 @@ import unittest
 from mock import Mock
 
 from pyramid_basemodel import Session
-from pyramid_simpleauth import model
+from pyramid_simpleauth import model, tree
+from pyramid_simpleauth.model import get_existing_user
 import transaction
 
-try: # pragma: no cover
+try:  # pragma: no cover
     from webtest import TestApp
-except ImportError: # pragma: no cover
+except ImportError:  # pragma: no cover
     pass
 
+
 def config_factory(**settings):
-    """Call with settings to make and configure a configurator instance, binding
-      to an in memory db.
+    """Call with settings to make and configure a configurator instance,
+      binding to an in memory db.
     """
-    
+
     from pyramid.config import Configurator
     from pyramid.session import UnencryptedCookieSessionFactoryConfig
-    
+
     # Patch the settings to use an in memory db for testing, which should
     # be dropped every time the app is created.
     settings['sqlalchemy.url'] = 'sqlite:///:memory:'
@@ -42,26 +44,26 @@ class BaseTestCase(unittest.TestCase):
 
     def setUp(self):
         """Configure the Pyramid application."""
-        
+
         # Configure redirect routes
         view_names = ('change_password', 'change_username', 'confirm_email',
                       'prefer_email')
         settings = dict(('simpleauth.after_%s_route' % view_name,
                          'success_path') for view_name in view_names)
         self.config = config_factory(**settings)
-        # Add routes for change_password, change_username, confirm_email_address
-        # and preferred_email
+        # Add routes for change_password, change_username,
+        # confirm_email_address and preferred_email
         self.config.add_route('success_path', 'victory_path')
         self.app = TestApp(self.config.make_wsgi_app())
-    
+
     def tearDown(self):
         """Make sure the session is cleared between tests."""
-        
+
         Session.remove()
-    
+
     def makeUser(self, username, password):
         """Create and save a user with the credentials provided."""
-        
+
         user = model.User()
         user.username = username
         user.password = model.encrypt(password)
@@ -72,35 +74,35 @@ class BaseTestCase(unittest.TestCase):
 
     def makeUserWithEmail(self):
         "Helper method that creates a user with an email"
-        user = self.makeUser(u'thruflo', u'password')
+        user = self.makeUser(u'thruflo', u'Password')
         Session.add(user)
         user.emails.append(model.Email(address=u'foo@example.com'))
         transaction.commit()
         Session.add(user)
         return user
 
-    def authenticate(self, user):
+    def authenticate(self, **post_data):
         "Authenticate user"
-        post_data = {
-            'username': 'thruflo',
-            'password': 'password'
-        }
+        if not post_data:
+            post_data = {
+                'username': 'thruflo',
+                'password': 'Password'
+            }
         headers = {'X-Requested-With': 'XMLHttpRequest'}
         return self.app.post('/auth/authenticate', post_data, headers=headers)
 
 
 class TestSignup(BaseTestCase):
-    
+
     def test_render_signup_form(self):
         """A GET request to the signup view should render the signup form."""
-        
+
         res = self.app.get('/auth/signup')
         self.failUnless('<input id="username" name="username"' in res.body)
-    
+
     def test_signup(self):
         """Signup saves a user and their email address."""
-        
-        from pyramid_simpleauth.model import get_existing_user
+
         # Sanity check there isn't an existing user.
         existing = get_existing_user(username='thruflo')
         self.assertTrue(existing is None)
@@ -108,19 +110,20 @@ class TestSignup(BaseTestCase):
         post_data = {
             'username': 'user',
             'email': 'foo@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
+        assert res  # to satisfy pyflakes
         # Now there is a user.
         existing = get_existing_user(username='user')
         self.assertTrue(existing is not None)
         # And their email address is...
         self.assertTrue(existing.emails[0].address == 'foo@gmail.com')
-    
+
     def test_signup_remember(self):
         """Signup logs the user in."""
-        
+
         # The first request sets an empty session cookie.
         res = self.app.post('/auth/signup', {}, status="*")
         self.assertTrue(len(res.headers['Set-Cookie']) < 200)
@@ -128,21 +131,21 @@ class TestSignup(BaseTestCase):
         post_data = {
             'username': 'user',
             'email': 'foo@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
         self.assertTrue(len(res.headers['Set-Cookie']) > 250)
-    
+
     def test_signup_redirect(self):
         """Signup redirects to the user's profile page."""
-        
+
         # Signup.
         post_data = {
             'username': 'thruflo',
             'email': 'foo@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
         # The response redirects to `/` by default with no routes or settings.
@@ -154,11 +157,12 @@ class TestSignup(BaseTestCase):
         post_data = {
             'username': 'thruflo2',
             'email': 'foo+2@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
-        self.assertTrue(res.headers['Location'] == 'http://localhost/some/path')
+        self.assertTrue(
+            res.headers['Location'] == 'http://localhost/some/path')
         # The response redirects to the `simpleauth.after_signup_route` route
         # if specified.
         settings = {
@@ -170,15 +174,15 @@ class TestSignup(BaseTestCase):
         post_data = {
             'username': 'thruflo3',
             'email': 'foo+3@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
         self.assertTrue(res.headers['Location'] == 'http://localhost/wob')
-    
+
     def test_signup_event(self):
         """Signup fires a ``UserSignedUp`` event."""
-        
+
         from pyramid_simpleauth.events import UserSignedUp
         from pyramid_simpleauth.model import User
         # Setup event listener.
@@ -190,36 +194,38 @@ class TestSignup(BaseTestCase):
         post_data = {
             'username': 'jo',
             'email': 'foo@gmail.com',
-            'password': 'password',
-            'confirm': 'password'
+            'password': 'Password',
+            'confirm': 'Password'
         }
         res = self.app.post('/auth/signup', post_data, status=302)
+        assert res  # to satisfy pyflakes
         # Handler was called with the authentiated user as the second arg.
         self.assertTrue(mock_subscriber.called)
         event = mock_subscriber.call_args_list[0][0][0]
         self.assertTrue(isinstance(event.user, User))
-    
+
 
 class TestLogin(BaseTestCase):
-    
+
     def test_render_login_form(self):
         """A GET request to the login view should render the login form."""
-        
+
         res = self.app.get('/auth/login')
         self.failUnless('<input id="username" name="username"' in res.body)
-    
+
     def test_render_login_next(self):
-        """A GET request to the login view will pass through the `next` param."""
-        
+        """A GET request to the login view will pass through the `next` param.
+        """
+
         res = self.app.get('/auth/login?next=/foo/bar')
         tag = '<input id="next" name="next" type="hidden" value="/foo/bar" />'
         self.failUnless(tag in res.body)
-    
+
     def test_login(self):
         """Login remembers the user."""
-        
+
         # Create a user.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         # Login with the wrong details sets an empty session cookie.
         post_data = {
             'username': 'pooplo',
@@ -227,23 +233,24 @@ class TestLogin(BaseTestCase):
         }
         res = self.app.post('/auth/login', post_data, status="*")
         self.assertTrue(len(res.headers['Set-Cookie']) < 200)
-        # Login with the right details remembers the user in the session cookie.
+        # Login with the right details remembers the user in the session
+        # cookie.
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/login', post_data, status="*")
         self.assertTrue(len(res.headers['Set-Cookie']) > 250)
-    
+
     def test_login_redirect(self):
         """login redirects to ``next``."""
-        
+
         # Create a user.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         # Login with the right details redirects to `next`.
         post_data = {
             'username': 'thruflo',
-            'password': 'password',
+            'password': 'Password',
             'next': '/foo/bar'
         }
         res = self.app.post('/auth/login', post_data, status=302)
@@ -251,7 +258,7 @@ class TestLogin(BaseTestCase):
         # If `next` is not provided, redirects to `/`
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/login', post_data, status=302)
         self.assertTrue(res.headers['Location'] == 'http://localhost/')
@@ -261,7 +268,8 @@ class TestLogin(BaseTestCase):
         self.app = TestApp(self.config.make_wsgi_app())
         res = self.app.post('/auth/login', post_data, status=302)
         self.assertEquals(res.location, 'http://localhost/some/path')
-        # Or the `simpleauth.after_logout_route` route if specified and exposed.
+        # Or the `simpleauth.after_logout_route` route if specified and
+        # exposed.
         settings = {
             'simpleauth.after_login_route': 'flobble'
         }
@@ -270,10 +278,10 @@ class TestLogin(BaseTestCase):
         self.app = TestApp(self.config.make_wsgi_app())
         res = self.app.post('/auth/login', post_data, status=302)
         self.assertTrue(res.headers['Location'] == 'http://localhost/wob')
-    
+
     def test_login_event(self):
         """Login fires a ``UserLoggedIn`` event."""
-        
+
         from pyramid_simpleauth.events import UserLoggedIn
         from pyramid_simpleauth.model import User
         # Setup event listener.
@@ -282,63 +290,64 @@ class TestLogin(BaseTestCase):
         self.config.add_subscriber(mock_subscriber, UserLoggedIn)
         self.app = TestApp(self.config.make_wsgi_app())
         # Login.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/login', post_data, status=302)
+        assert res  # to satisfy pyflakes
         # Handler was called with the authentiated user as the second arg.
         self.assertTrue(mock_subscriber.called)
         event = mock_subscriber.call_args_list[0][0][0]
         self.assertTrue(isinstance(event.user, User))
-    
+
 
 class TestAuthenticate(BaseTestCase):
-    
+
     def test_authenticate_requires_xhr(self):
         """Authenticate must be called with an XMLHTTPRequest."""
-        
+
         # Authentication remembers the user in the session cookie.
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/authenticate', post_data, status=404)
         self.assertTrue('The resource could not be found.' in res.body)
-    
+
     def test_authenticate_remembers(self):
         """Authenticate remembers the user."""
-        
+
         # Create a user.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         # Authentication remembers the user in the session cookie.
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         headers = {'X-Requested-With': 'XMLHttpRequest'}
         res = self.app.post('/auth/authenticate', post_data, headers=headers)
         self.assertTrue(len(res.headers['Set-Cookie']) > 250)
-    
+
     def test_authenticate_returns_json(self):
         """Authenticate returns the user's data in JSON format."""
-        
+
         import json
         # Create a user.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         # Returns JSON.
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         headers = {'X-Requested-With': 'XMLHttpRequest'}
         res = self.app.post('/auth/authenticate', post_data, headers=headers)
         self.assertTrue(json.loads(res.body)['username'] == 'thruflo')
-    
+
     def test_authenticate_logged_in_event(self):
         """Authenticate fires a ``UserLoggedIn`` event."""
-        
+
         from pyramid_simpleauth.events import UserLoggedIn
         from pyramid_simpleauth.model import User
         # Setup event listener.
@@ -347,49 +356,51 @@ class TestAuthenticate(BaseTestCase):
         self.config.add_subscriber(mock_subscriber, UserLoggedIn)
         self.app = TestApp(self.config.make_wsgi_app())
         # Login.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         headers = {'X-Requested-With': 'XMLHttpRequest'}
         res = self.app.post('/auth/authenticate', post_data, headers=headers)
+        assert res  # to satisfy pyflakes
         # Handler was called with the authentiated user as the second arg.
         self.assertTrue(mock_subscriber.called)
         event = mock_subscriber.call_args_list[0][0][0]
         self.assertTrue(isinstance(event.user, User))
-    
+
 
 class TestLogout(BaseTestCase):
-    
+
     def test_logout(self):
         """Logout forgets the user."""
-        
+
         # Create a user.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         # Login.
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/login', post_data, status=302)
         self.assertTrue(len(res.headers['Set-Cookie']) > 250)
         # Logout.
-        res = self.app.get('/auth/logout', status=302)
+        res = self.app.post('/auth/logout', status=302)
         self.assertTrue(len(res.headers['Set-Cookie']) < 200)
-    
+
     def test_logout_redirects(self):
         """Logout redirects."""
-        
+
         # The response redirects to `/` by default with no routes or settings.
-        res = self.app.get('/auth/logout', status=302)
+        res = self.app.post('/auth/logout', status=302)
         self.assertTrue(res.headers['Location'] == 'http://localhost/')
         # The response redirects to the `index` route if exposed.
         self.config = config_factory()
         self.config.add_route('index', 'some/path')
         self.app = TestApp(self.config.make_wsgi_app())
-        res = self.app.get('/auth/logout', status=302)
-        self.assertTrue(res.headers['Location'] == 'http://localhost/some/path')
+        res = self.app.post('/auth/logout', status=302)
+        self.assertTrue(
+            res.headers['Location'] == 'http://localhost/some/path')
         # The response redirects to the `simpleauth.after_logout_route` route
         # if specified.
         settings = {
@@ -398,12 +409,12 @@ class TestLogout(BaseTestCase):
         self.config = config_factory(**settings)
         self.config.add_route('flobble', 'wob')
         self.app = TestApp(self.config.make_wsgi_app())
-        res = self.app.get('/auth/logout', status=302)
+        res = self.app.post('/auth/logout', status=302)
         self.assertTrue(res.headers['Location'] == 'http://localhost/wob')
-    
+
     def test_loggedout_event(self):
         """Logout fires a ``UserLoggedOut`` event."""
-        
+
         from pyramid_simpleauth.events import UserLoggedOut
         from pyramid_simpleauth.model import User
         # Setup event listener.
@@ -412,24 +423,25 @@ class TestLogout(BaseTestCase):
         self.config.add_subscriber(mock_subscriber, UserLoggedOut)
         self.app = TestApp(self.config.make_wsgi_app())
         # Login.
-        self.makeUser('thruflo', 'password')
+        self.makeUser('thruflo', 'Password')
         post_data = {
             'username': 'thruflo',
-            'password': 'password'
+            'password': 'Password'
         }
         res = self.app.post('/auth/login', post_data, status=302)
         # Logout.
-        res = self.app.get('/auth/logout', status=302)
+        res = self.app.post('/auth/logout', status=302)
+        assert res  # to satisfy pyflakes
         # Handler was called with the authentiated user as the second arg.
         self.assertTrue(mock_subscriber.called)
         event = mock_subscriber.call_args_list[0][0][0]
         self.assertTrue(isinstance(event.user, User))
-    
+
     def test_loggedout_event_requires_user(self):
         """Logout only fires a ``UserLoggedOut`` event when there was an
           authenticated user.
         """
-        
+
         from pyramid_simpleauth.events import UserLoggedOut
         # Setup event listener.
         mock_subscriber = Mock()
@@ -437,10 +449,11 @@ class TestLogout(BaseTestCase):
         self.config.add_subscriber(mock_subscriber, UserLoggedOut)
         self.app = TestApp(self.config.make_wsgi_app())
         # Logout.
-        res = self.app.get('/auth/logout', status=302)
+        res = self.app.post('/auth/logout', status=302)
+        assert res  # to satisfy pyflakes
         # Handler was not called.
         self.assertFalse(mock_subscriber.called)
-    
+
 
 class TestChangePassword(BaseTestCase):
 
@@ -448,11 +461,11 @@ class TestChangePassword(BaseTestCase):
         "No password change if old password is not corret"
 
         # Create a user.
-        user = self.makeUser('thruflo', 'password')
+        user = self.makeUser('thruflo', 'Password')
         Session.add(user)
         old_hash = user.password
 
-        self.authenticate(user)
+        self.authenticate()
 
         # Attempt to change password.
         post_data = {
@@ -476,13 +489,13 @@ class TestChangePassword(BaseTestCase):
         from pyramid import testing
 
         # Create a user.
-        user = self.makeUser('thruflo', 'password')
+        user = self.makeUser('thruflo', 'Password')
         Session.add(user)
 
         post_data = {
             'old_password': 'foobarbaz',
-            'new_password': 'swordpas',
-            'new_confirm':  'swordpas',
+            'new_password': 'sworDpas',
+            'new_confirm':  'sworDpas',
         }
         request = testing.DummyRequest(post=post_data)
         request.user = user
@@ -491,20 +504,19 @@ class TestChangePassword(BaseTestCase):
 
         self.assertTrue(res['user'])
 
-
     def test_new_passwords_dont_match(self):
         "No password change if new passwords don't match"
 
         # Create a user.
-        user = self.makeUser('thruflo', 'password')
+        user = self.makeUser('thruflo', 'Password')
         Session.add(user)
         old_hash = user.password
 
-        self.authenticate(user)
+        self.authenticate()
 
         # Attempt to change password.
         post_data = {
-            'old_password': 'password',
+            'old_password': 'Password',
             'new_password': 'swordpas',
             'new_confirm':  'oswdrpsa',
         }
@@ -520,17 +532,17 @@ class TestChangePassword(BaseTestCase):
         "If all conditions are met, change password"
 
         # Create a user.
-        user = self.makeUser('thruflo', 'password')
+        user = self.makeUser('thruflo', 'Password')
         Session.add(user)
         old_hash = user.password
 
-        self.authenticate(user)
+        self.authenticate()
 
         # Attempt to change password.
         post_data = {
-            'old_password': 'password',
-            'new_password': 'swordpas',
-            'new_confirm':  'swordpas',
+            'old_password': 'Password',
+            'new_password': 'sworDpas',
+            'new_confirm':  'sworDpas',
             'next':         '/foo/bar',
         }
         res = self.app.post('/auth/change_password', post_data)
@@ -549,14 +561,14 @@ class TestChangeUsername(BaseTestCase):
     def test_success(self):
         "Change username with valid input"
         # Create a user.
-        user = self.makeUser(u'thruflo', u'password')
+        user = self.makeUser(u'thruflo', u'Password')
 
         # Attempt to change username
         post_data = {
             'username': u'bob',
             'next':     '/foo/bar',
         }
-        self.authenticate(user)
+        self.authenticate()
         res = self.app.post('/auth/change_username', post_data)
 
         # Verify redirect
@@ -569,14 +581,14 @@ class TestChangeUsername(BaseTestCase):
     def test_failure(self):
         "Change username with invalid input"
         # Create a user.
-        user = self.makeUser(u'thruflo', u'password')
+        user = self.makeUser(u'thruflo', u'Password')
 
         # Attempt to assign bogus username
         post_data = {
             'username': u'$ @ 88 , /',
             'next':     '/foo/bar',
         }
-        self.authenticate(user)
+        self.authenticate()
         res = self.app.post('/auth/change_username', post_data)
 
         # Verify response body
@@ -647,7 +659,6 @@ class TestConfirmEmailAddress(BaseTestCase):
         res = self.app.get(url)
         self.assertTrue('invalid' in res.body)
 
-
         # Verify that email address has been confirmed
         Session.add(email)
         Session.refresh(email)
@@ -674,7 +685,7 @@ class TestPreferEmail(BaseTestCase):
         self.assertEquals(user.preferred_email, email2)
 
         # Attempt to make the address primary
-        self.authenticate(user)
+        self.authenticate()
         self.app.post('/auth/prefer_email', {
             'email_address': email1.address
         })
@@ -692,7 +703,7 @@ class TestPreferEmail(BaseTestCase):
         email = user.emails[0]
 
         # Attempt to make the address primary
-        self.authenticate(user)
+        self.authenticate()
         self.app.post('/auth/prefer_email', {
             'email_address': email.address + 'not an email address'
         })
@@ -729,3 +740,83 @@ class TestPreferEmail(BaseTestCase):
         # Verify that the new email is now the preferred email
         Session.add(user)
         self.assertEquals(user.preferred_email.address, u'bar@example.com')
+
+
+class TestDeleteUser(BaseTestCase):
+
+    def add_user_root(self):
+        "Configure app with /users/<username> route"
+        self.config = config_factory()
+        self.config.add_route('users', 'users/*traverse',
+                              factory=tree.UserRoot, use_global_views=True)
+        self.app = TestApp(self.config.make_wsgi_app())
+
+    def test_success(self):
+        "User can delete itself"
+        self.add_user_root()
+
+        user = self.makeUser('thruflo', 'Password')
+        Session.add(user)
+
+        self.authenticate()
+
+        # Attempt to delete user
+        res = self.app.get('/users/thruflo/delete_user')
+
+        # Verify confirmation message
+        self.assertTrue('Are you really sure' in res.body)
+
+        # Verify that the user has not yet been deleted
+        self.assertIsNotNone(get_existing_user(username='thruflo'))
+
+        # Delete the user
+        res = self.app.post('/users/thruflo/delete_user')
+
+        # Verify that the user has now been deleted
+        self.assertIsNone(get_existing_user(username='thruflo'))
+
+        # User should be logged out
+        self.assertTrue(len(res.headers['Set-Cookie']) < 200)
+
+    def test_other_user(self):
+        "Non-admin user cannot delete other user"
+        self.add_user_root()
+
+        # User to delete
+        self.makeUser('alice', 'Password')
+
+        # Login as other user
+        bob = self.makeUser('bob', 'Password')
+        model.save(bob)
+        transaction.commit()
+        self.authenticate(username='bob', password='Password')
+
+        # Try to delete user
+        res = self.app.post('/users/alice/delete_user', status=403)
+
+        # Verify that the user has not been deleted
+        self.assertIsNotNone(get_existing_user(username='alice'))
+        # User should still be logged in
+        self.assertGreater(len(res.headers['Set-Cookie']), 250)
+
+    def test_admin(self):
+        "Admin should be allowed to delete any user"
+        self.add_user_root()
+
+        # User to delete
+        self.makeUser('alice', 'Password')
+
+        # Login as admin
+        admin = self.makeUser('admin', 'Password')
+        admin.roles.append(model.Role(name='admin'))
+        model.save(admin)
+        transaction.commit()
+        self.authenticate(username='admin', password='Password')
+
+        # Delete user
+        res = self.app.post('/users/alice/delete_user')
+
+        # Verify that user has been successfully deleted
+        self.assertIsNone(get_existing_user(username='alice'))
+        # Admin should still be logged in
+        self.assertGreater(len(res.headers['Set-Cookie']), 250)
